package bus_service

import (
	"errors"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/go-admin-team/go-admin-core/sdk/service"
	"go-admin/app/admin/models"
	"go-admin/app/admin/models/bus_models"
	"go-admin/tools"
)

type ProductService struct {
	service.Service
}

// GetProductList 获取产品列表
func (e *ProductService) GetProductList(c *gin.Context, req bus_models.ProductListReq) (bus_models.ProductListResp, error) {
	var products []bus_models.BusProduct
	var resp bus_models.ProductListResp

	// 构建查询条件，假设我们支持根据 product_code 查询
	db := e.Orm.WithContext(c.Request.Context())

	var productIDs []uint64
	// 用户权限校验：非管理员仅能查看自己门店的数据
	if !(tools.GetRoleName(c) == "admin" || tools.GetRoleName(c) == "系统管理员") {
		sysUser, err := models.GetSysUserByCtx(c)
		if err != nil {
			return resp, errors.New("操作失败: " + err.Error())
		}
		fmt.Println("sysUser:", sysUser)

		// 查询合作商信息（根据 userId 关联）
		var cooperative bus_models.BusCooperative
		err = e.Orm.Where("user_id = ?", sysUser.UserId).First(&cooperative).Error
		if err != nil {
			return resp, errors.New("获取合作商信息失败: " + err.Error())
		}

		// 查询合作商绑定的产品 ID 列表
		if err = e.Orm.Model(&bus_models.BusCooperativeProduct{}).
			Where("cooperative_id = ?", cooperative.ID).
			Pluck("product_id", &productIDs).Error; err != nil {
			return resp, errors.New("获取合作商产品信息失败: " + err.Error())
		}

		// 如果该合作商没有绑定任何产品，直接返回空数据
		if len(productIDs) == 0 {
			resp.List = []bus_models.BusProduct{}
			resp.Total = 0
			resp.Page = req.Page
			resp.PageSize = req.PageSize
			return resp, nil
		}

		// 限制查询仅限这些产品
		db = db.Where("id IN ?", productIDs)
	}

	page := req.Page - 1
	if page < 0 {
		page = 0
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	// 如果需要支持分页，可以根据请求条件调整查询
	if req.ProductCode != "" {
		db = db.Where("product_code = ?", req.ProductCode)
	}
	if req.ProductName != "" {
		db = db.Where("product_name LIKE ?", "%"+req.ProductName+"%")
	}
	if req.ProductType != 0 {
		db = db.Where("product_type = ?", req.ProductType)
	}
	if req.Province != "" {
		db = db.Where("province = ?", req.Province)
	}
	if req.City != "" {
		db = db.Where("city = ?", req.City)
	}
	if req.Platform != 0 {
		db = db.Where("platform = ?", req.Platform)
	}

	// 查询数据库
	if err := db.Offset(page * req.Page).Limit(req.PageSize).Find(&products).Error; err != nil {
		return resp, errors.New("查询产品列表失败")
	}

	resp.List = products
	resp.Total = len(products)
	resp.Page = page + 1
	resp.PageSize = req.PageSize

	return resp, nil
}

// CreateProduct 新建产品
func (e *ProductService) CreateProduct(req bus_models.CreateProductReq) (bus_models.CreateProductResp, error) {
	var resp bus_models.CreateProductResp
	product := bus_models.BusProduct{
		ProductCode: req.ProductCode,
		ServiceCode: req.ServiceCode,
		ProductName: req.ProductName,
		ProductAttr: req.ProductAttr,
		ProductType: req.ProductType,
		Type:        req.Type,
		Size:        req.Size,
		Price:       req.Price,
		Discount:    req.Discount,
		RetailPrice: req.RetailPrice,
		Description: req.Description,
		Province:    req.Province,
		City:        req.City,
		Platform:    req.Platform,
	}

	// 将产品插入数据库
	if err := e.Orm.Create(&product).Error; err != nil {
		return resp, errors.New("创建产品失败")
	}

	// 返回创建成功的响应
	resp.ID = product.ID

	return resp, nil
}

// EditProduct 编辑产品
func (e *ProductService) EditProduct(req bus_models.EditProductReq) error {
	product := bus_models.BusProduct{
		ProductCode: req.ProductCode,
		ServiceCode: req.ServiceCode,
		ProductName: req.ProductName,
		ProductAttr: req.ProductAttr,
		ProductType: req.ProductType,
		Type:        req.Type,
		Size:        req.Size,
		Price:       req.Price,
		Discount:    req.Discount,
		RetailPrice: req.RetailPrice,
		Description: req.Description,
		Province:    req.Province,
		City:        req.City,
		Platform:    req.Platform,
	}

	// 更新产品信息
	if err := e.Orm.Model(&bus_models.BusProduct{}).Where("product_code = ?", req.ProductCode).
		Updates(&product).Error; err != nil {
		return errors.New("编辑产品失败")
	}

	return nil
}

// DeleteProduct 删除产品
func (e *ProductService) DeleteProduct(req bus_models.DeleteProductReq) error {
	// 执行删除操作
	if err := e.Orm.Where("id = ?", req.ID).Delete(&bus_models.BusProduct{}).Error; err != nil {
		return errors.New("删除产品失败")
	}

	return nil
}
